@@ -5,11 +5,38 @@ the flask extension
 from functools import wraps
 import logging
 
-from flask import request, current_app, g
+from flask import request, current_app, g, Blueprint
 
 from .errors import RateLimitExceeded, ConfigurationError
+from flask.ext.limiter.storage import storage_from_string
 from .strategies import STRATEGIES
-from .util import storage_from_string, parse_many, get_ipaddr
+from .util import parse_many, get_ipaddr
+
+class C:
+    ENABLED = "RATELIMIT_ENABLED"
+    HEADERS_ENABLED = "RATELIMIT_HEADERS_ENABLED"
+    STORAGE_URL = "RATELIMIT_STORAGE_URL"
+    STRATEGY = "RATELIMIT_STRATEGY"
+    GLOBAL_LIMITS = "RATELIMIT_GLOBAL"
+
+
+class ExtLimit(object):
+    """
+    simple wrapper to encapsulate limits and their context
+    """
+    def __init__(self, limit, key_func, scope, per_method):
+        self._limit = limit
+        self.key_func = key_func
+        self._scope = scope
+        self.per_method = per_method
+
+    @property
+    def limit(self):
+        return self._limit() if callable(self._limit) else self._limit
+
+    @property
+    def scope(self):
+        return self._scope(request.endpoint) if callable(self._scope) else self._scope
 
 class Limiter(object):
     """
@@ -34,7 +61,7 @@ class Limiter(object):
         self.app = app
         self.enabled = True
         self.global_limits = []
-        self.exempt_routes = []
+        self.exempt_routes = set()
         self.request_filters = []
         self.headers_enabled = headers_enabled
         self.strategy = strategy
@@ -42,11 +69,16 @@ class Limiter(object):
         for limit in global_limits:
             self.global_limits.extend(
                 [
-                    (key_func, limit, None) for limit in parse_many(limit)
+                    ExtLimit(
+                        limit, key_func, None, False
+                    ) for limit in parse_many(limit)
                 ]
             )
         self.route_limits = {}
         self.dynamic_route_limits = {}
+        self.blueprint_limits = {}
+        self.blueprint_dynamic_limits = {}
+        self.blueprint_exempt = set()
         self.storage = self.limiter = None
         self.key_func = key_func
         self.logger = logging.getLogger("flask-limiter")
@@ -61,32 +93,35 @@ class Limiter(object):
         """
         :param app: :class:`flask.Flask` instance to rate limit.
         """
-        self.enabled = app.config.setdefault("RATELIMIT_ENABLED", True)
+        self.enabled = app.config.setdefault(C.ENABLED, True)
         self.headers_enabled = (
             self.headers_enabled
-            or app.config.setdefault("RATELIMIT_HEADERS_ENABLED", False)
+            or app.config.setdefault(C.HEADERS_ENABLED, True)
         )
         self.storage = storage_from_string(
             self.storage_uri
-            or app.config.setdefault('RATELIMIT_STORAGE_URL', 'memory://')
+            or app.config.setdefault(C.STORAGE_URL, 'memory://')
         )
         strategy = (
             self.strategy
-            or app.config.setdefault('RATELIMIT_STRATEGY', 'fixed-window')
+            or app.config.setdefault(C.STRATEGY, 'fixed-window')
         )
         if strategy not in STRATEGIES:
             raise ConfigurationError("Invalid rate limiting strategy %s" % strategy)
         self.limiter = STRATEGIES[strategy](self.storage)
-        conf_limits = app.config.get("RATELIMIT_GLOBAL", None)
+        conf_limits = app.config.get(C.GLOBAL_LIMITS, None)
         if not self.global_limits and conf_limits:
             self.global_limits = [
-                (self.key_func, limit, None) for limit in parse_many(conf_limits)
+                ExtLimit(
+                    limit, self.key_func, None, False
+                ) for limit in parse_many(conf_limits)
             ]
         app.before_request(self.__check_request_limit)
         app.after_request(self.__inject_headers)
 
+        # purely for backward compatibility as stated in flask documentation
         if not hasattr(app, 'extensions'):
-            app.extensions = {}
+            app.extensions = {} # pragma: no cover
         app.extensions['limiter'] = self
 
     def __inject_headers(self, response):
@@ -114,10 +149,12 @@ class Limiter(object):
                 view_func.__module__, view_func.__name__
             ) if view_func else ""
         )
-        if (view_func == current_app.send_static_file
+        if (not request.endpoint
+            or view_func == current_app.send_static_file
             or name in self.exempt_routes
             or not self.enabled
             or any(fn() for fn in self.request_filters)
+            or request.blueprint in self.blueprint_exempt
         ):
             return
         limits = (
@@ -126,32 +163,54 @@ class Limiter(object):
         )
         dynamic_limits = []
         if name in self.dynamic_route_limits:
-            for key_func, limit_func, scope in self.dynamic_route_limits[name]:
+            for lim in self.dynamic_route_limits[name]:
                 try:
                     dynamic_limits.extend(
-                        [key_func, limit, scope] for limit in parse_many(limit_func())
+                        ExtLimit(
+                            limit, lim.key_func, lim.scope, lim.per_method
+                        ) for limit in parse_many(lim.limit)
                     )
                 except ValueError as e:
                     self.logger.error(
                         "failed to load ratelimit for view function %s (%s)"
                         , name, e
                     )
+        if request.blueprint:
+            if (request.blueprint in self.blueprint_dynamic_limits
+                and not dynamic_limits
+            ):
+                for lim in self.blueprint_dynamic_limits[request.blueprint]:
+                    try:
+                        dynamic_limits.extend(
+                            ExtLimit(
+                                limit, lim.key_func, lim.scope, lim.per_method
+                            ) for limit in parse_many(lim.limit)
+                        )
+                    except ValueError as e:
+                        self.logger.error(
+                            "failed to load ratelimit for blueprint %s (%s)"
+                            , request.blueprint, e
+                        )
+            if (request.blueprint in self.blueprint_limits
+                and not limits
+            ):
+               limits.extend(self.blueprint_limits[request.blueprint])
 
         failed_limit = None
         limit_for_header = None
-        for key_func, limit, scope in (limits + dynamic_limits or self.global_limits):
-            limit_scope = (
-                              scope if not callable(scope) else scope(endpoint)
-                          ) or endpoint
-            if not limit_for_header or limit < limit_for_header[0]:
-                limit_for_header = (limit, key_func(), limit_scope)
-            if not self.limiter.hit(limit, key_func(), limit_scope):
+        for lim in (limits + dynamic_limits or self.global_limits):
+            limit_scope = lim.scope or endpoint
+            if lim.per_method:
+                limit_scope += ":%s" % request.method
+            if not limit_for_header or lim.limit < limit_for_header[0]:
+                limit_for_header = (lim.limit, lim.key_func(), limit_scope)
+            if not self.limiter.hit(lim.limit, lim.key_func(), limit_scope):
                 self.logger.warn(
                     "ratelimit %s (%s) exceeded at endpoint: %s"
-                    , limit, key_func(), limit_scope
+                    , lim.limit, lim.key_func(), limit_scope
                 )
-                failed_limit = limit
-                limit_for_header = (limit, key_func(), limit_scope)
+                failed_limit = lim.limit
+                limit_for_header = (lim.limit, lim.key_func(), limit_scope)
         g.view_rate_limit = limit_for_header
 
         if failed_limit:
@@ -159,44 +218,66 @@ class Limiter(object):
 
     def __limit_decorator(self, limit_value,
                           key_func=None, shared=False,
-                          scope=None):
+                          scope=None,
+                          per_method=False):
         _scope = scope if shared else None
 
-        def _inner(fn):
-            name = "%s.%s" % (fn.__module__, fn.__name__)
-
-            @wraps(fn)
-            def __inner(*a, **k):
-                return fn(*a, **k)
+        def _inner(obj):
             func = key_func or self.key_func
+            is_route = not isinstance(obj, Blueprint)
+            name = "%s.%s" % (obj.__module__, obj.__name__) if is_route else obj.name
+            dynamic_limit, static_limits = None, []
             if callable(limit_value):
-                self.dynamic_route_limits.setdefault(name, []).append(
-                    (func, limit_value, _scope)
-                )
+                dynamic_limit = ExtLimit(limit_value, func, _scope, per_method)
             else:
                 try:
-                    self.route_limits.setdefault(name, []).extend(
-                        [(func, limit, _scope) for limit in parse_many(limit_value)]
-                    )
+                    static_limits = [ExtLimit(
+                        limit, func, _scope, per_method
+                    ) for limit in parse_many(limit_value)]
                 except ValueError as e:
                     self.logger.error(
-                        "failed to configure view function %s (%s)", name, e
+                        "failed to configure %s %s (%s)",
+                        "view function" if is_route else "blueprint", name, e
                     )
-            return __inner
+            if not isinstance(obj, Blueprint):
+                @wraps(obj)
+                def __inner(*a, **k):
+                    return obj(*a, **k)
+                if dynamic_limit:
+                    self.dynamic_route_limits.setdefault(name, []).append(
+                        dynamic_limit
+                    )
+                else:
+                    self.route_limits.setdefault(name, []).extend(
+                        static_limits
+                    )
+                return __inner
+            else:
+                if dynamic_limit:
+                    self.blueprint_dynamic_limits.setdefault(name, []).append(
+                        dynamic_limit
+                    )
+                else:
+                    self.blueprint_limits.setdefault(name, []).extend(
+                        static_limits
+                    )
+
         return _inner
 
 
-    def limit(self, limit_value, key_func=None):
+    def limit(self, limit_value, key_func=None, per_method=False):
         """
-        decorator to be used for rate limiting individual routes.
+        decorator to be used for rate limiting individual routes or blueprints.
 
         :param limit_value: rate limit string or a callable that returns a string.
          :ref:`ratelimit-string` for more details.
-        :param key_func: function/lambda to extract the unique identifier for
+        :param function key_func: function/lambda to extract the unique identifier for
          the rate limit. defaults to remote address of the request.
+        :param bool per_method: whether the limit is sub categorized into the http
+         method of the request.
         :return:
         """
-        return self.__limit_decorator(limit_value, key_func)
+        return self.__limit_decorator(limit_value, key_func, per_method=per_method)
 
 
     def shared_limit(self, limit_value, scope, key_func=None):
@@ -207,23 +288,26 @@ class Limiter(object):
          :ref:`ratelimit-string` for more details.
         :param scope: a string or callable that returns a string
          for defining the rate limiting scope.
-        :param key_func: function/lambda to extract the unique identifier for
+        :param function key_func: function/lambda to extract the unique identifier for
          the rate limit. defaults to remote address of the request.
         """
         return self.__limit_decorator(limit_value, key_func, True, scope)
 
 
 
-    def exempt(self, fn):
+    def exempt(self, obj):
         """
-        decorator to mark a view as exempt from rate limits.
+        decorator to mark a view or all views in a route as exempt from rate limits.
         """
-        name = "%s.%s" % (fn.__module__, fn.__name__)
-        @wraps(fn)
-        def __inner(*a, **k):
-            return fn(*a, **k)
-        self.exempt_routes.append(name)
-        return __inner
+        if not isinstance(obj, Blueprint):
+            name = "%s.%s" % (obj.__module__, obj.__name__)
+            @wraps(obj)
+            def __inner(*a, **k):
+                return obj(*a, **k)
+            self.exempt_routes.add(name)
+            return __inner
+        else:
+            self.blueprint_exempt.add(obj.name)
 
     def request_filter(self, fn):
         """
